@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2018 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -34,8 +34,10 @@ import sys
 import os.path
 import logging
 import weakref
+
+import six
+
 from silx.gui import qt
-from silx.third_party import six
 from .SafeFileIconProvider import SafeFileIconProvider
 
 _logger = logging.getLogger(__name__)
