@@ -26,7 +26,7 @@
 
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "05/10/2018"
+__date__ = "08/03/2019"
 
 
 import unittest
@@ -36,16 +36,8 @@ import shutil
 import os
 import io
 import weakref
-
-try:
-    import fabio
-except ImportError:
-    fabio = None
-try:
-    import h5py
-except ImportError:
-    h5py = None
-
+import fabio
+import h5py
 import silx.io.url
 from silx.gui import qt
 from silx.gui.utils import testutils
@@ -62,36 +54,33 @@ def setUpModule():
     data = numpy.arange(100 * 100)
     data.shape = 100, 100
 
-    if fabio is not None:
-        filename = _tmpDirectory + "/singleimage.edf"
-        image = fabio.edfimage.EdfImage(data=data)
-        image.write(filename)
+    filename = _tmpDirectory + "/singleimage.edf"
+    image = fabio.edfimage.EdfImage(data=data)
+    image.write(filename)
 
-    if h5py is not None:
-        filename = _tmpDirectory + "/data.h5"
-        f = h5py.File(filename, "w")
-        f["scalar"] = 10
-        f["image"] = data
-        f["cube"] = [data, data + 1, data + 2]
-        f["complex_image"] = data * 1j
-        f["group/image"] = data
-        f["nxdata/foo"] = 10
-        f["nxdata"].attrs["NX_class"] = u"NXdata"
-        f.close()
+    filename = _tmpDirectory + "/data.h5"
+    f = h5py.File(filename, "w")
+    f["scalar"] = 10
+    f["image"] = data
+    f["cube"] = [data, data + 1, data + 2]
+    f["complex_image"] = data * 1j
+    f["group/image"] = data
+    f["nxdata/foo"] = 10
+    f["nxdata"].attrs["NX_class"] = u"NXdata"
+    f.close()
 
-    if h5py is not None:
-        directory = os.path.join(_tmpDirectory, "data")
-        os.mkdir(directory)
-        filename = os.path.join(directory, "data.h5")
-        f = h5py.File(filename, "w")
-        f["scalar"] = 10
-        f["image"] = data
-        f["cube"] = [data, data + 1, data + 2]
-        f["complex_image"] = data * 1j
-        f["group/image"] = data
-        f["nxdata/foo"] = 10
-        f["nxdata"].attrs["NX_class"] = u"NXdata"
-        f.close()
+    directory = os.path.join(_tmpDirectory, "data")
+    os.mkdir(directory)
+    filename = os.path.join(directory, "data.h5")
+    f = h5py.File(filename, "w")
+    f["scalar"] = 10
+    f["image"] = data
+    f["cube"] = [data, data + 1, data + 2]
+    f["complex_image"] = data * 1j
+    f["group/image"] = data
+    f["nxdata/foo"] = 10
+    f["nxdata"].attrs["NX_class"] = u"NXdata"
+    f.close()
 
     filename = _tmpDirectory + "/badformat.h5"
     with io.open(filename, "wb") as f:
@@ -141,7 +130,7 @@ class _UtilsMixin(object):
         path2_ = os.path.normcase(path2)
         if path1_ == path2_:
             # Use the unittest API to log and display error
-            self.assertNotEquals(path1, path2)
+            self.assertNotEqual(path1, path2)
 
 
 class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
@@ -185,8 +174,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertEqual(dialog.result(), qt.QDialog.Rejected)
 
     def testSelectRoot_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -211,8 +198,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertEqual(dialog.result(), qt.QDialog.Accepted)
 
     def testSelectGroup_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -243,8 +228,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertEqual(dialog.result(), qt.QDialog.Accepted)
 
     def testSelectDataset_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -275,8 +258,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertEqual(dialog.result(), qt.QDialog.Accepted)
 
     def testClickOnBackToParentTool(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -307,8 +288,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(url.text(), _tmpDirectory)
 
     def testClickOnBackToRootTool(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -332,8 +311,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         # self.assertFalse(button.isEnabled())
 
     def testClickOnBackToDirectoryTool(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -361,8 +338,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.allowedLeakingWidgets = 1
 
     def testClickOnHistoryTools(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -402,8 +377,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(url.text(), path3)
 
     def testSelectImageFromEdf(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -412,13 +385,11 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         filename = _tmpDirectory + "/singleimage.edf"
         url = silx.io.url.DataUrl(scheme="silx", file_path=filename, data_path="/scan_0/instrument/detector_0/data")
         dialog.selectUrl(url.path())
-        self.assertTrue(dialog._selectedData().shape, (100, 100))
+        self.assertEqual(dialog._selectedData().shape, (100, 100))
         self.assertSamePath(dialog.selectedFile(), filename)
         self.assertSamePath(dialog.selectedUrl(), url.path())
 
     def testSelectImage(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -428,13 +399,11 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         path = silx.io.url.DataUrl(scheme="silx", file_path=filename, data_path="/image").path()
         dialog.selectUrl(path)
         # test
-        self.assertTrue(dialog._selectedData().shape, (100, 100))
+        self.assertEqual(dialog._selectedData().shape, (100, 100))
         self.assertSamePath(dialog.selectedFile(), filename)
         self.assertSamePath(dialog.selectedUrl(), path)
 
     def testSelectScalar(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -449,8 +418,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(dialog.selectedUrl(), path)
 
     def testSelectGroup(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -467,8 +434,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(uri.data_path(), "/group")
 
     def testSelectRoot(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -485,8 +450,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(uri.data_path(), "/")
 
     def testSelectH5_Activate(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -516,11 +479,12 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.qWaitForPendingActions(dialog)
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         filename = _tmpDirectory + "/badformat.h5"
-        index = browser.rootIndex().model().index(filename)
+        index = browser.model().index(filename)
+        browser.selectIndex(index)
         browser.activated.emit(index)
         self.qWaitForPendingActions(dialog)
         # test
-        self.assertTrue(dialog.selectedUrl(), filename)
+        self.assertSamePath(dialog.selectedUrl(), filename)
 
     def _countSelectableItems(self, model, rootIndex):
         selectable = 0
@@ -533,10 +497,6 @@ class TestDataFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         return selectable
 
     def testFilterExtensions(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -558,8 +518,6 @@ class TestDataFileDialog_FilterDataset(testutils.TestCaseQt, _UtilsMixin):
         return dialog
 
     def testSelectGroup_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -585,8 +543,6 @@ class TestDataFileDialog_FilterDataset(testutils.TestCaseQt, _UtilsMixin):
         self.assertFalse(button.isEnabled())
 
     def testSelectDataset_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -632,8 +588,6 @@ class TestDataFileDialog_FilterGroup(testutils.TestCaseQt, _UtilsMixin):
         return dialog
 
     def testSelectGroup_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -666,8 +620,6 @@ class TestDataFileDialog_FilterGroup(testutils.TestCaseQt, _UtilsMixin):
         self.assertRaises(Exception, dialog.selectedData)
 
     def testSelectDataset_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -711,8 +663,6 @@ class TestDataFileDialog_FilterNXdata(testutils.TestCaseQt, _UtilsMixin):
         return dialog
 
     def testSelectGroupRefused_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -740,8 +690,6 @@ class TestDataFileDialog_FilterNXdata(testutils.TestCaseQt, _UtilsMixin):
         self.assertRaises(Exception, dialog.selectedData)
 
     def testSelectNXdataAccepted_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         dialog.show()
@@ -906,7 +854,7 @@ class TestDataFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         dialog2 = self.createDialog()
         result = dialog2.restoreState(state)
         self.assertTrue(result)
-        self.assertNotEquals(dialog2.directory(), directory)
+        self.assertNotEqual(dialog2.directory(), directory)
 
     def testHistory(self):
         dialog = self.createDialog()
@@ -944,8 +892,6 @@ class TestDataFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         self.assertIsNone(dialog._selectedData())
 
     def testBadSubpath(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         self.qWaitForPendingActions(dialog)
 
@@ -965,8 +911,6 @@ class TestDataFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         self.assertEqual(url.data_path(), "/group")
 
     def testUnsupportedSlicingPath(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         self.qWaitForPendingActions(dialog)
         dialog.selectUrl(_tmpDirectory + "/data.h5?path=/cube&slice=0")
