@@ -26,7 +26,7 @@
 
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "05/10/2018"
+__date__ = "08/03/2019"
 
 
 import unittest
@@ -36,16 +36,8 @@ import shutil
 import os
 import io
 import weakref
-
-try:
-    import fabio
-except ImportError:
-    fabio = None
-try:
-    import h5py
-except ImportError:
-    h5py = None
-
+import fabio
+import h5py
 import silx.io.url
 from silx.gui import qt
 from silx.gui.utils import testutils
@@ -63,42 +55,39 @@ def setUpModule():
     data = numpy.arange(100 * 100)
     data.shape = 100, 100
 
-    if fabio is not None:
-        filename = _tmpDirectory + "/singleimage.edf"
-        image = fabio.edfimage.EdfImage(data=data)
-        image.write(filename)
+    filename = _tmpDirectory + "/singleimage.edf"
+    image = fabio.edfimage.EdfImage(data=data)
+    image.write(filename)
 
-        filename = _tmpDirectory + "/multiframe.edf"
-        image = fabio.edfimage.EdfImage(data=data)
-        image.appendFrame(data=data + 1)
-        image.appendFrame(data=data + 2)
-        image.write(filename)
+    filename = _tmpDirectory + "/multiframe.edf"
+    image = fabio.edfimage.EdfImage(data=data)
+    image.appendFrame(data=data + 1)
+    image.appendFrame(data=data + 2)
+    image.write(filename)
 
-        filename = _tmpDirectory + "/singleimage.msk"
-        image = fabio.fit2dmaskimage.Fit2dMaskImage(data=data % 2 == 1)
-        image.write(filename)
+    filename = _tmpDirectory + "/singleimage.msk"
+    image = fabio.fit2dmaskimage.Fit2dMaskImage(data=data % 2 == 1)
+    image.write(filename)
 
-    if h5py is not None:
-        filename = _tmpDirectory + "/data.h5"
-        f = h5py.File(filename, "w")
+    filename = _tmpDirectory + "/data.h5"
+    with h5py.File(filename, "w") as f:
         f["scalar"] = 10
         f["image"] = data
         f["cube"] = [data, data + 1, data + 2]
+        f["single_frame"] = [data + 5]
         f["complex_image"] = data * 1j
         f["group/image"] = data
-        f.close()
 
-    if h5py is not None:
-        directory = os.path.join(_tmpDirectory, "data")
-        os.mkdir(directory)
-        filename = os.path.join(directory, "data.h5")
-        f = h5py.File(filename, "w")
+    directory = os.path.join(_tmpDirectory, "data")
+    os.mkdir(directory)
+    filename = os.path.join(directory, "data.h5")
+    with h5py.File(filename, "w") as f:
         f["scalar"] = 10
         f["image"] = data
         f["cube"] = [data, data + 1, data + 2]
+        f["single_frame"] = [data + 5]
         f["complex_image"] = data * 1j
         f["group/image"] = data
-        f.close()
 
     filename = _tmpDirectory + "/badformat.edf"
     with io.open(filename, "wb") as f:
@@ -148,7 +137,7 @@ class _UtilsMixin(object):
         path2_ = os.path.normcase(path2)
         if path1_ == path2_:
             # Use the unittest API to log and display error
-            self.assertNotEquals(path1, path2)
+            self.assertNotEqual(path1, path2)
 
 
 class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
@@ -192,8 +181,6 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertEqual(dialog.result(), qt.QDialog.Rejected)
 
     def testDisplayAndClickOpen(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -259,8 +246,6 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertEqual(dialog.viewMode(), qt.QFileDialog.List)
 
     def testClickOnBackToParentTool(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -291,8 +276,6 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(url.text(), _tmpDirectory)
 
     def testClickOnBackToRootTool(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -316,8 +299,6 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         # self.assertFalse(button.isEnabled())
 
     def testClickOnBackToDirectoryTool(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -345,8 +326,6 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.allowedLeakingWidgets = 1
 
     def testClickOnHistoryTools(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -386,8 +365,6 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(url.text(), path3)
 
     def testSelectImageFromEdf(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -396,14 +373,12 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         filename = _tmpDirectory + "/singleimage.edf"
         path = filename
         dialog.selectUrl(path)
-        self.assertTrue(dialog.selectedImage().shape, (100, 100))
+        self.assertEqual(dialog.selectedImage().shape, (100, 100))
         self.assertSamePath(dialog.selectedFile(), filename)
         path = silx.io.url.DataUrl(scheme="fabio", file_path=filename).path()
         self.assertSamePath(dialog.selectedUrl(), path)
 
     def testSelectImageFromEdf_Activate(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -421,13 +396,11 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         browser.activated.emit(index)
         self.qWaitForPendingActions(dialog)
         # test
-        self.assertTrue(dialog.selectedImage().shape, (100, 100))
+        self.assertEqual(dialog.selectedImage().shape, (100, 100))
         self.assertSamePath(dialog.selectedFile(), filename)
         self.assertSamePath(dialog.selectedUrl(), path)
 
     def testSelectFrameFromEdf(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -438,14 +411,12 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         dialog.selectUrl(path)
         # test
         image = dialog.selectedImage()
-        self.assertTrue(image.shape, (100, 100))
-        self.assertTrue(image[0, 0], 1)
+        self.assertEqual(image.shape, (100, 100))
+        self.assertEqual(image[0, 0], 1)
         self.assertSamePath(dialog.selectedFile(), filename)
         self.assertSamePath(dialog.selectedUrl(), path)
 
     def testSelectImageFromMsk(self):
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -455,13 +426,11 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         path = silx.io.url.DataUrl(scheme="fabio", file_path=filename).path()
         dialog.selectUrl(path)
         # test
-        self.assertTrue(dialog.selectedImage().shape, (100, 100))
+        self.assertEqual(dialog.selectedImage().shape, (100, 100))
         self.assertSamePath(dialog.selectedFile(), filename)
         self.assertSamePath(dialog.selectedUrl(), path)
 
     def testSelectImageFromH5(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -471,13 +440,11 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         path = silx.io.url.DataUrl(scheme="silx", file_path=filename, data_path="/image").path()
         dialog.selectUrl(path)
         # test
-        self.assertTrue(dialog.selectedImage().shape, (100, 100))
+        self.assertEqual(dialog.selectedImage().shape, (100, 100))
         self.assertSamePath(dialog.selectedFile(), filename)
         self.assertSamePath(dialog.selectedUrl(), path)
 
     def testSelectH5_Activate(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -498,8 +465,6 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(dialog.selectedUrl(), path)
 
     def testSelectFrameFromH5(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.show()
         self.qWaitForWindowExposed(dialog)
@@ -509,8 +474,23 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         path = silx.io.url.DataUrl(scheme="silx", file_path=filename, data_path="/cube", data_slice=(1, )).path()
         dialog.selectUrl(path)
         # test
-        self.assertTrue(dialog.selectedImage().shape, (100, 100))
-        self.assertTrue(dialog.selectedImage()[0, 0], 1)
+        self.assertEqual(dialog.selectedImage().shape, (100, 100))
+        self.assertEqual(dialog.selectedImage()[0, 0], 1)
+        self.assertSamePath(dialog.selectedFile(), filename)
+        self.assertSamePath(dialog.selectedUrl(), path)
+
+    def testSelectSingleFrameFromH5(self):
+        dialog = self.createDialog()
+        dialog.show()
+        self.qWaitForWindowExposed(dialog)
+
+        # init state
+        filename = _tmpDirectory + "/data.h5"
+        path = silx.io.url.DataUrl(scheme="silx", file_path=filename, data_path="/single_frame", data_slice=(0, )).path()
+        dialog.selectUrl(path)
+        # test
+        self.assertEqual(dialog.selectedImage().shape, (100, 100))
+        self.assertEqual(dialog.selectedImage()[0, 0], 5)
         self.assertSamePath(dialog.selectedFile(), filename)
         self.assertSamePath(dialog.selectedUrl(), path)
 
@@ -524,11 +504,12 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         self.qWaitForPendingActions(dialog)
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         filename = _tmpDirectory + "/badformat.edf"
-        index = browser.rootIndex().model().index(filename)
+        index = browser.model().index(filename)
+        browser.selectIndex(index)
         browser.activated.emit(index)
         self.qWaitForPendingActions(dialog)
         # test
-        self.assertTrue(dialog.selectedUrl(), filename)
+        self.assertSamePath(dialog.selectedUrl(), filename)
 
     def _countSelectableItems(self, model, rootIndex):
         selectable = 0
@@ -541,10 +522,6 @@ class TestImageFileDialogInteraction(testutils.TestCaseQt, _UtilsMixin):
         return selectable
 
     def testFilterExtensions(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
-        if fabio is None:
-            self.skipTest("fabio is missing")
         dialog = self.createDialog()
         browser = testutils.findChildren(dialog, qt.QWidget, name="browser")[0]
         filters = testutils.findChildren(dialog, qt.QWidget, name="fileTypeCombo")[0]
@@ -588,7 +565,7 @@ class TestImageFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         result = dialog2.restoreState(state)
         self.qWaitForPendingActions(dialog2)
         self.assertTrue(result)
-        self.assertTrue(dialog2.colormap().getNormalization(), "log")
+        self.assertEqual(dialog2.colormap().getNormalization(), "log")
 
     def printState(self):
         """
@@ -685,7 +662,7 @@ class TestImageFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         result = dialog.restoreState(state)
         self.assertTrue(result)
         colormap = dialog.colormap()
-        self.assertTrue(colormap.getNormalization(), "log")
+        self.assertEqual(colormap.getNormalization(), "log")
 
     def testRestoreRobusness(self):
         """What's happen if you try to open a config file with a different
@@ -711,7 +688,7 @@ class TestImageFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         dialog2 = self.createDialog()
         result = dialog2.restoreState(state)
         self.assertTrue(result)
-        self.assertNotEquals(dialog2.directory(), directory)
+        self.assertNotEqual(dialog2.directory(), directory)
 
     def testHistory(self):
         dialog = self.createDialog()
@@ -745,16 +722,12 @@ class TestImageFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         self.assertSamePath(dialog.directory(), _tmpDirectory)
 
     def testBadDataType(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.selectUrl(_tmpDirectory + "/data.h5::/complex_image")
         self.qWaitForPendingActions(dialog)
         self.assertIsNone(dialog._selectedData())
 
     def testBadDataShape(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         dialog.selectUrl(_tmpDirectory + "/data.h5::/unknown")
         self.qWaitForPendingActions(dialog)
@@ -773,8 +746,6 @@ class TestImageFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         self.assertIsNone(dialog._selectedData())
 
     def testBadSubpath(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         self.qWaitForPendingActions(dialog)
 
@@ -794,8 +765,6 @@ class TestImageFileDialogApi(testutils.TestCaseQt, _UtilsMixin):
         self.assertEqual(url.data_path(), "/group")
 
     def testBadSlicingPath(self):
-        if h5py is None:
-            self.skipTest("h5py is missing")
         dialog = self.createDialog()
         self.qWaitForPendingActions(dialog)
         dialog.selectUrl(_tmpDirectory + "/data.h5::/cube[a;45,-90]")
