@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2019 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -29,15 +29,19 @@ The :class:`PlotWindow` is a subclass of :class:`.PlotWidget`.
 
 __authors__ = ["V.A. Sole", "T. Vincent"]
 __license__ = "MIT"
-__date__ = "12/10/2018"
+__date__ = "12/04/2019"
 
-import collections
+try:
+    from collections import abc
+except ImportError:  # Python2 support
+    import collections as abc
 import logging
 import weakref
 
 import silx
 from silx.utils.weakref import WeakMethodProxy
 from silx.utils.deprecation import deprecated
+from silx.utils.proxy import docstring
 
 from . import PlotWidget
 from . import actions
@@ -118,7 +122,7 @@ class PlotWindow(PlotWidget):
         self._curvesROIDockWidget = None
         self._maskToolsDockWidget = None
         self._consoleDockWidget = None
-        self._statsWidget = None
+        self._statsDockWidget = None
 
         # Create color bar, hidden by default for backward compatibility
         self._colorbar = ColorBarWidget(parent=self, plot=self)
@@ -128,53 +132,53 @@ class PlotWindow(PlotWidget):
         self.group.setExclusive(False)
 
         self.resetZoomAction = self.group.addAction(
-            actions.control.ResetZoomAction(self))
+            actions.control.ResetZoomAction(self, parent=self))
         self.resetZoomAction.setVisible(resetzoom)
         self.addAction(self.resetZoomAction)
 
-        self.zoomInAction = actions.control.ZoomInAction(self)
+        self.zoomInAction = actions.control.ZoomInAction(self, parent=self)
         self.addAction(self.zoomInAction)
 
-        self.zoomOutAction = actions.control.ZoomOutAction(self)
+        self.zoomOutAction = actions.control.ZoomOutAction(self, parent=self)
         self.addAction(self.zoomOutAction)
 
         self.xAxisAutoScaleAction = self.group.addAction(
-            actions.control.XAxisAutoScaleAction(self))
+            actions.control.XAxisAutoScaleAction(self, parent=self))
         self.xAxisAutoScaleAction.setVisible(autoScale)
         self.addAction(self.xAxisAutoScaleAction)
 
         self.yAxisAutoScaleAction = self.group.addAction(
-            actions.control.YAxisAutoScaleAction(self))
+            actions.control.YAxisAutoScaleAction(self, parent=self))
         self.yAxisAutoScaleAction.setVisible(autoScale)
         self.addAction(self.yAxisAutoScaleAction)
 
         self.xAxisLogarithmicAction = self.group.addAction(
-            actions.control.XAxisLogarithmicAction(self))
+            actions.control.XAxisLogarithmicAction(self, parent=self))
         self.xAxisLogarithmicAction.setVisible(logScale)
         self.addAction(self.xAxisLogarithmicAction)
 
         self.yAxisLogarithmicAction = self.group.addAction(
-            actions.control.YAxisLogarithmicAction(self))
+            actions.control.YAxisLogarithmicAction(self, parent=self))
         self.yAxisLogarithmicAction.setVisible(logScale)
         self.addAction(self.yAxisLogarithmicAction)
 
         self.gridAction = self.group.addAction(
-            actions.control.GridAction(self, gridMode='both'))
+            actions.control.GridAction(self, gridMode='both', parent=self))
         self.gridAction.setVisible(grid)
         self.addAction(self.gridAction)
 
         self.curveStyleAction = self.group.addAction(
-            actions.control.CurveStyleAction(self))
+            actions.control.CurveStyleAction(self, parent=self))
         self.curveStyleAction.setVisible(curveStyle)
         self.addAction(self.curveStyleAction)
 
         self.colormapAction = self.group.addAction(
-            actions.control.ColormapAction(self))
+            actions.control.ColormapAction(self, parent=self))
         self.colormapAction.setVisible(colormap)
         self.addAction(self.colormapAction)
 
         self.colorbarAction = self.group.addAction(
-            actions_control.ColorBarAction(self, self))
+            actions_control.ColorBarAction(self, parent=self))
         self.colorbarAction.setVisible(False)
         self.addAction(self.colorbarAction)
         self._colorbar.setVisible(False)
@@ -194,18 +198,18 @@ class PlotWindow(PlotWidget):
         self.getMaskAction().setVisible(mask)
 
         self._intensityHistoAction = self.group.addAction(
-            actions_histogram.PixelIntensitiesHistoAction(self))
+            actions_histogram.PixelIntensitiesHistoAction(self, parent=self))
         self._intensityHistoAction.setVisible(False)
 
         self._medianFilter2DAction = self.group.addAction(
-            actions_medfilt.MedianFilter2DAction(self))
+            actions_medfilt.MedianFilter2DAction(self, parent=self))
         self._medianFilter2DAction.setVisible(False)
 
         self._medianFilter1DAction = self.group.addAction(
-            actions_medfilt.MedianFilter1DAction(self))
+            actions_medfilt.MedianFilter1DAction(self, parent=self))
         self._medianFilter1DAction.setVisible(False)
 
-        self.fitAction = self.group.addAction(actions_fit.FitAction(self))
+        self.fitAction = self.group.addAction(actions_fit.FitAction(self, parent=self))
         self.fitAction.setVisible(fit)
         self.addAction(self.fitAction)
 
@@ -217,10 +221,8 @@ class PlotWindow(PlotWidget):
 
         # Make colorbar background white
         self._colorbar.setAutoFillBackground(True)
-        palette = self._colorbar.palette()
-        palette.setColor(qt.QPalette.Background, qt.Qt.white)
-        palette.setColor(qt.QPalette.Window, qt.Qt.white)
-        self._colorbar.setPalette(palette)
+        self._sigAxesVisibilityChanged.connect(self._updateColorBarBackground)
+        self._updateColorBarBackground()
 
         gridLayout = qt.QGridLayout()
         gridLayout.setSpacing(0)
@@ -252,7 +254,7 @@ class PlotWindow(PlotWidget):
                 hbox.addWidget(self.controlButton)
 
             if position:  # Add PositionInfo widget to the bottom of the plot
-                if isinstance(position, collections.Iterable):
+                if isinstance(position, abc.Iterable):
                     # Use position as a set of converters
                     converters = position
                 else:
@@ -280,7 +282,7 @@ class PlotWindow(PlotWidget):
             parent=self, plot=self)
         self.addToolBar(self._interactiveModeToolBar)
 
-        self._toolbar = self._createToolBar(title='Plot', parent=None)
+        self._toolbar = self._createToolBar(title='Plot', parent=self)
         self.addToolBar(self._toolbar)
 
         self._outputToolBar = tools.OutputToolBar(parent=self, plot=self)
@@ -294,6 +296,40 @@ class PlotWindow(PlotWidget):
             for action in toolbar.actions():
                 self.addAction(action)
 
+    @docstring(PlotWidget)
+    def setBackgroundColor(self, color):
+        super(PlotWindow, self).setBackgroundColor(color)
+        self._updateColorBarBackground()
+
+    @docstring(PlotWidget)
+    def setDataBackgroundColor(self, color):
+        super(PlotWindow, self).setDataBackgroundColor(color)
+        self._updateColorBarBackground()
+
+    @docstring(PlotWidget)
+    def setForegroundColor(self, color):
+        super(PlotWindow, self).setForegroundColor(color)
+        self._updateColorBarBackground()
+
+    def _updateColorBarBackground(self):
+        """Update the colorbar background according to the state of the plot"""
+        if self._isAxesDisplayed():
+            color = self.getBackgroundColor()
+        else:
+            color = self.getDataBackgroundColor()
+            if not color.isValid():
+                # If no color defined, use the background one
+                color = self.getBackgroundColor()
+
+        foreground = self.getForegroundColor()
+
+        palette = self._colorbar.palette()
+        palette.setColor(qt.QPalette.Background, color)
+        palette.setColor(qt.QPalette.Window, color)
+        palette.setColor(qt.QPalette.WindowText, foreground)
+        palette.setColor(qt.QPalette.Text, foreground)
+        self._colorbar.setPalette(palette)
+
     def getInteractiveModeToolBar(self):
         """Returns QToolBar controlling interactive mode.
 
@@ -449,6 +485,22 @@ class PlotWindow(PlotWidget):
             self.tabifyDockWidget(self._dockWidgets[0],
                                   dock_widget)
 
+    def __handleFirstDockWidgetShow(self, visible):
+        """Handle QDockWidget.visibilityChanged
+
+        It calls :meth:`addTabbedDockWidget` for the `sender` widget.
+        This allows to call `addTabbedDockWidget` lazily.
+
+        It disconnect itself from the signal once done.
+
+        :param bool visible:
+        """
+        if visible:
+            dockWidget = self.sender()
+            dockWidget.visibilityChanged.disconnect(
+                self.__handleFirstDockWidgetShow)
+            self.addTabbedDockWidget(dockWidget)
+
     def getColorBarWidget(self):
         """Returns the embedded :class:`ColorBarWidget` widget.
 
@@ -457,24 +509,16 @@ class PlotWindow(PlotWidget):
         return self._colorbar
 
     # getters for dock widgets
-    @property
-    @deprecated(replacement="getLegendsDockWidget()", since_version="0.4.0")
-    def legendsDockWidget(self):
-        return self.getLegendsDockWidget()
 
     def getLegendsDockWidget(self):
         """DockWidget with Legend panel"""
         if self._legendsDockWidget is None:
             self._legendsDockWidget = LegendsDockWidget(plot=self)
             self._legendsDockWidget.hide()
-            self.addTabbedDockWidget(self._legendsDockWidget)
+            self._legendsDockWidget.visibilityChanged.connect(
+                self.__handleFirstDockWidgetShow)
         return self._legendsDockWidget
 
-    @property
-    @deprecated(replacement="getCurvesRoiWidget()", since_version="0.4.0")
-    def curvesROIDockWidget(self):
-        return self.getCurvesRoiDockWidget()
-
     def getCurvesRoiDockWidget(self):
         # Undocumented for a "soft deprecation" in version 0.7.0
         # (still used internally for lazy loading)
@@ -482,7 +526,8 @@ class PlotWindow(PlotWidget):
             self._curvesROIDockWidget = CurvesROIDockWidget(
                 plot=self, name='Regions Of Interest')
             self._curvesROIDockWidget.hide()
-            self.addTabbedDockWidget(self._curvesROIDockWidget)
+            self._curvesROIDockWidget.visibilityChanged.connect(
+                self.__handleFirstDockWidgetShow)
         return self._curvesROIDockWidget
 
     def getCurvesRoiWidget(self):
@@ -496,18 +541,14 @@ class PlotWindow(PlotWidget):
         """
         return self.getCurvesRoiDockWidget().roiWidget
 
-    @property
-    @deprecated(replacement="getMaskToolsDockWidget()", since_version="0.4.0")
-    def maskToolsDockWidget(self):
-        return self.getMaskToolsDockWidget()
-
     def getMaskToolsDockWidget(self):
         """DockWidget with image mask panel (lazy-loaded)."""
         if self._maskToolsDockWidget is None:
             self._maskToolsDockWidget = MaskToolsDockWidget(
                 plot=self, name='Mask')
             self._maskToolsDockWidget.hide()
-            self.addTabbedDockWidget(self._maskToolsDockWidget)
+            self._maskToolsDockWidget.visibilityChanged.connect(
+                self.__handleFirstDockWidgetShow)
         return self._maskToolsDockWidget
 
     def getStatsWidget(self):
@@ -515,16 +556,18 @@ class PlotWindow(PlotWidget):
 
         :rtype: BasicStatsWidget
         """
-        if self._statsWidget is None:
-            dockWidget = qt.QDockWidget(parent=self)
-            dockWidget.setWindowTitle("Curves stats")
-            dockWidget.layout().setContentsMargins(0, 0, 0, 0)
-            self._statsWidget = BasicStatsWidget(parent=self, plot=self)
-            self._statsWidget.sigVisibilityChanged.connect(self.getStatsAction().setChecked)
-            dockWidget.setWidget(self._statsWidget)
-            dockWidget.hide()
-            self.addTabbedDockWidget(dockWidget)
-        return self._statsWidget
+        if self._statsDockWidget is None:
+            self._statsDockWidget = qt.QDockWidget()
+            self._statsDockWidget.setWindowTitle("Curves stats")
+            self._statsDockWidget.layout().setContentsMargins(0, 0, 0, 0)
+            statsWidget = BasicStatsWidget(parent=self, plot=self)
+            self._statsDockWidget.setWidget(statsWidget)
+            statsWidget.sigVisibilityChanged.connect(
+                self.getStatsAction().setChecked)
+            self._statsDockWidget.hide()
+            self._statsDockWidget.visibilityChanged.connect(
+                self.__handleFirstDockWidgetShow)
+        return self._statsDockWidget.widget()
 
     # getters for actions
     @property
@@ -539,11 +582,6 @@ class PlotWindow(PlotWidget):
     def panModeAction(self):
         return self.getInteractiveModeToolBar().getPanModeAction()
 
-    @property
-    @deprecated(replacement="getConsoleAction()", since_version="0.4.0")
-    def consoleAction(self):
-        return self.getConsoleAction()
-
     def getConsoleAction(self):
         """QAction handling the IPython console activation.
 
@@ -563,11 +601,6 @@ class PlotWindow(PlotWidget):
                 self._consoleAction.setEnabled(False)
         return self._consoleAction
 
-    @property
-    @deprecated(replacement="getCrosshairAction()", since_version="0.4.0")
-    def crosshairAction(self):
-        return self.getCrosshairAction()
-
     def getCrosshairAction(self):
         """Action toggling crosshair cursor mode.
 
@@ -577,11 +610,6 @@ class PlotWindow(PlotWidget):
             self._crosshairAction = actions.control.CrosshairAction(self, color='red')
         return self._crosshairAction
 
-    @property
-    @deprecated(replacement="getMaskAction()", since_version="0.4.0")
-    def maskAction(self):
-        return self.getMaskAction()
-
     def getMaskAction(self):
         """QAction toggling image mask dock widget
 
@@ -589,12 +617,6 @@ class PlotWindow(PlotWidget):
         """
         return self.getMaskToolsDockWidget().toggleViewAction()
 
-    @property
-    @deprecated(replacement="getPanWithArrowKeysAction()",
-                since_version="0.4.0")
-    def panWithArrowKeysAction(self):
-        return self.getPanWithArrowKeysAction()
-
     def getPanWithArrowKeysAction(self):
         """Action toggling pan with arrow keys.
 
@@ -604,11 +626,6 @@ class PlotWindow(PlotWidget):
             self._panWithArrowKeysAction = actions.control.PanWithArrowKeysAction(self)
         return self._panWithArrowKeysAction
 
-    @property
-    @deprecated(replacement="getRoiAction()", since_version="0.4.0")
-    def roiAction(self):
-        return self.getRoiAction()
-
     def getStatsAction(self):
         if self._statsAction is None:
             self._statsAction = qt.QAction('Curves stats', self)
@@ -829,7 +846,9 @@ class Plot2D(PlotWindow):
         posInfo = [
             ('X', lambda x, y: x),
             ('Y', lambda x, y: y),
-            ('Data', WeakMethodProxy(self._getImageValue))]
+            ('Data', WeakMethodProxy(self._getImageValue)),
+            ('Dims', WeakMethodProxy(self._getImageDims)),
+        ]
 
         super(Plot2D, self).__init__(parent=parent, backend=backend,
                                      resetzoom=True, autoScale=False,
@@ -929,6 +948,15 @@ class Plot2D(PlotWindow):
             return value, "Masked"
         return value
 
+    def _getImageDims(self, *args):
+        activeImage = self.getActiveImage()
+        if (activeImage is not None and
+                    activeImage.getData(copy=False) is not None):
+            dims = activeImage.getData(copy=False).shape[1::-1]
+            return 'x'.join(str(dim) for dim in dims)
+        else:
+            return '-'
+
     def getProfileToolbar(self):
         """Profile tools attached to this plot
 
