@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2015-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2015-2019 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -60,6 +60,91 @@ class ProgramFunction(object):
         pass
 
 
+class Fog(event.Notifier, ProgramFunction):
+    """Linear fog over the whole scene content.
+
+    The background of the viewport is used as fog color,
+    otherwise it defaults to white.
+    """
+    # TODO: add more controls (set fog range), add more fog modes
+
+    _fragDecl = """
+    /* (1/(far - near) or 0, near) z in [0 (camera), -inf[ */
+    uniform vec2 fogExtentInfo;
+
+    /* Color to use as fog color */
+    uniform vec3 fogColor;
+
+    vec4 fog(vec4 color, vec4 cameraPosition) {
+        /* d = (pos - near) / (far - near) */
+        float distance = fogExtentInfo.x * (cameraPosition.z/cameraPosition.w - fogExtentInfo.y);
+        float fogFactor = clamp(distance, 0.0, 1.0);
+        vec3 rgb = mix(color.rgb, fogColor, fogFactor);
+        return vec4(rgb.r, rgb.g, rgb.b, color.a);
+    }
+    """
+
+    _fragDeclNoop = """
+    vec4 fog(vec4 color, vec4 cameraPosition) {
+        return color;
+    }
+    """
+
+    def __init__(self):
+        super(Fog, self).__init__()
+        self._isOn = True
+
+    @property
+    def isOn(self):
+        """True to enable fog, False to disable (bool)"""
+        return self._isOn
+
+    @isOn.setter
+    def isOn(self, isOn):
+        isOn = bool(isOn)
+        if self._isOn != isOn:
+            self._isOn = bool(isOn)
+            self.notify()
+
+    @property
+    def fragDecl(self):
+        return self._fragDecl if self.isOn else self._fragDeclNoop
+
+    @property
+    def fragCall(self):
+        return "fog"
+
+    @staticmethod
+    def _zExtentCamera(viewport):
+        """Return (far, near) planes Z in camera coordinates.
+
+        :param Viewport viewport:
+        :return: (far, near) position in camera coords (from 0 to -inf)
+        """
+        # Provide scene z extent in camera coords
+        bounds = viewport.camera.extrinsic.transformBounds(
+            viewport.scene.bounds(transformed=True, dataBounds=True))
+        return bounds[:, 2]
+
+    def setupProgram(self, context, program):
+        if not self.isOn:
+            return
+
+        far, near = context.cache(key='zExtentCamera',
+                                  factory=self._zExtentCamera,
+                                  viewport=context.viewport)
+        extent = far - near
+        gl.glUniform2f(program.uniforms['fogExtentInfo'],
+                       0.9/extent if extent != 0. else 0.,
+                       near)
+
+        # Use background color as fog color
+        bgColor = context.viewport.background
+        if bgColor is None:
+            bgColor = 1., 1., 1.
+        gl.glUniform3f(program.uniforms['fogColor'], *bgColor[:3])
+
+
 class ClippingPlane(ProgramFunction):
     """Description of a clipping plane and rendering.
 
